package de.kebidge.middleware.frontend;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.reactive.function.client.WebClient;

import reactor.core.publisher.Mono;

@Controller
public class ThymeleafController {

    @Value("${welcome.user}")
    private String user;

    @Value("${shoppingitem.endpoint}")
    private String shoppingItemEndpoint;

    @GetMapping("/")
    public String displayPage(Model model) {

        ShoppingItem[] items = WebClient
            .create(shoppingItemEndpoint)
            .get()
            .retrieve()
            .bodyToMono(ShoppingItem[].class)
            .block();

        model.addAttribute("user", user);
        model.addAttribute("items", items);
        return "page";
    }

    @PostMapping("/")
    public String addItem(@RequestParam String newitem, @RequestParam int amount, Model model) {

        ShoppingItem item = new ShoppingItem();
        item.item = newitem;
        item.amount = amount;

        WebClient
            .create(shoppingItemEndpoint + "add")
            .post()
            .body(Mono.just(item), ShoppingItem.class)
            .retrieve()
            .bodyToMono(ShoppingItem.class)
            .block();

        return "redirect:/";
    }

    @PostMapping("/delete")
    public String deleteItem(@RequestParam String item, Model model) {

        WebClient
            .create(shoppingItemEndpoint + "del-item/" + item)
            .delete()
            .retrieve()
            .bodyToMono(Void.class)      // dont expect a response
            .block();

        return "redirect:/";
    }

    @PostMapping("/delete-all")
    public String deleteItemById(@RequestParam Long id, Model model) {

        WebClient
            .create(shoppingItemEndpoint + "del-id/" + id)
            .delete()
            .retrieve()
            .bodyToMono(Void.class)      // dont expect a response
            .block();

        return "redirect:/";
    }

    @PostMapping("/delete-list")
    public String deleteItemList(Model model) {

        WebClient
            .create(shoppingItemEndpoint + "del-all")
            .delete()
            .retrieve()
            .bodyToMono(Void.class)      // dont expect a response
            .block();

        return "redirect:/";
    }
    
}